
#
# GENERATED WITH PDL::PP! Don't modify!
#
package PDL::NDBin::Actions_PP;

our @EXPORT_OK = qw(PDL::PP _flatten_into PDL::PP _flatten_into_grid PDL::PP _setnulltobad PDL::PP _icount_loop PDL::PP _imax_loop PDL::PP _imin_loop PDL::PP _isum_loop PDL::PP _iavg_loop PDL::PP _istddev_loop PDL::PP _istddev_post );
our %EXPORT_TAGS = (Func=>[@EXPORT_OK]);

use PDL::Core;
use PDL::Exporter;
use DynaLoader;



   our $VERSION = '0.020';
   our @ISA = ( 'PDL::Exporter','DynaLoader' );
   push @PDL::Core::PP, __PACKAGE__;
   bootstrap PDL::NDBin::Actions_PP $VERSION;


BEGIN {

# ABSTRACT: XS actions for PDL::NDBin

=head1 NAME

PDL::NDBin::Actions_PP - XS actions for PDL::NDBin

=head1 DESCRIPTION

This module contains the implementation of the core loop of the action classes
in the PDL::NDBin::Action namespace. The subroutines are not intended to be
called by the user.

=cut

# silence Test::Pod::Coverage warning
=for Pod::Coverage set_boundscheck set_debugging

=cut



}





=head1 FUNCTIONS



=cut






=head2 _flatten_into

=for sig

  Signature: (in(m); indx b(m); indx [o] idx(m); double step; double min; PDL_Indx n)

=for ref

Bin a series of values and flatten into an existing list of indices.

=for bad

_flatten_into processes bad values.
It will set the bad-value flag of all output ndarrays if the flag is set for any of the input ndarrays.


=cut






*_flatten_into = \&PDL::_flatten_into;





=head2 _flatten_into_grid

=for sig

  Signature: (in(); indx b(); double edge(n); indx [o] idx())

=for ref

Bin a series of values on a specified grid and flatten into an existing list of indices.

=for bad

_flatten_into_grid processes bad values.
It will set the bad-value flag of all output ndarrays if the flag is set for any of the input ndarrays.


=cut






*_flatten_into_grid = \&PDL::_flatten_into_grid;





=head2 _setnulltobad

=for sig

  Signature: (in(); indx count(); [o] out())

=for ref

Set empty bins to the bad value.

=for bad

_setnulltobad processes bad values.
The state of the bad-value flag of the output ndarrays is unknown.


=cut






*_setnulltobad = \&PDL::_setnulltobad;





=head2 _icount_loop

=for sig

  Signature: (in(n); indx idx(n); indx [o] out(m); PDL_Indx msize => m)

Count the number of elements in each bin.

This function returns a piddle of type I<indx> if you have a 64-bit-capable
PDL, otherwise it returns a piddle of type I<long>.

=for bad

_icount_loop processes bad values.
It will set the bad-value flag of all output ndarrays if the flag is set for any of the input ndarrays.


=cut






*_icount_loop = \&PDL::_icount_loop;





=head2 _imax_loop

=for sig

  Signature: (in(n); indx idx(n); [o] out(m); PDL_Indx msize => m)

=for ref

Find the maximum in each bin.

=for bad

_imax_loop processes bad values.
It will set the bad-value flag of all output ndarrays if the flag is set for any of the input ndarrays.


=cut






*_imax_loop = \&PDL::_imax_loop;





=head2 _imin_loop

=for sig

  Signature: (in(n); indx idx(n); [o] out(m); PDL_Indx msize => m)

=for ref

Find the minimum in each bin.

=for bad

_imin_loop processes bad values.
It will set the bad-value flag of all output ndarrays if the flag is set for any of the input ndarrays.


=cut






*_imin_loop = \&PDL::_imin_loop;





=head2 _isum_loop

=for sig

  Signature: (in(n); indx idx(n); int+ [o] out(m); indx [t] count(m); PDL_Indx msize => m)

Sum the elements in each bin.

This function returns a piddle of type I<int> or higher, to reduce the risk of
overflow when collecting sums.

=for bad

_isum_loop processes bad values.
It will set the bad-value flag of all output ndarrays if the flag is set for any of the input ndarrays.


=cut






*_isum_loop = \&PDL::_isum_loop;





=head2 _iavg_loop

=for sig

  Signature: (in(n); indx idx(n); double [o] out(m); indx [t] count(m); PDL_Indx msize => m)

Compute the average of the elements in each bin.

Credit for the algorithm goes to
L<I<ashawley> on commandlinefu.com|http://www.commandlinefu.com/commands/view/3437/compute-running-average-for-a-column-of-numbers>:

	awk '{ avg += ($1 - avg) / NR } END { print avg }'

This is a wonderful solution solving many of the problems with more naive
implementations:

=over 4

=item 1.

It's numerically well-behaved: out() is always of the order of magnitude of the
values themselves, unlike the sum of the values, which grows very large as the
number of elements grows large.

=item 2.

The subtraction in() - out() guarantees that the computation will be done in
the correct type (i.e., I<double> instead of the type of the input).

=item 3.

Requires only one temporary.

=item 4.

Requires only one pass over the data.

=back

I used to give the output array type I<float+>, but that creates more problems
than it solves. So now, averages are always computed in type I<double>. Besides
being the default type in PDL and the 'natural' floating-point type in C, it
also makes the implementation easier.

=for bad

_iavg_loop processes bad values.
It will set the bad-value flag of all output ndarrays if the flag is set for any of the input ndarrays.


=cut






*_iavg_loop = \&PDL::_iavg_loop;





=head2 _istddev_loop

=for sig

  Signature: (in(n); indx idx(n); double [o] out(m); indx [t] count(m); double [t] avg(m); PDL_Indx msize => m)

Compute the standard deviation of the elements in each bin.

Note that we compute the sample standard deviation, I<not> an estimate of the
population standard deviation (which differs by a factor).

Credit for the algorithm goes to
L<I<ashawley> on commandlinefu.com|http://www.commandlinefu.com/commands/view/3442/display-the-standard-deviation-of-a-column-of-numbers-with-awk>;

	awk '{ delta = $1 - avg; avg += delta / NR; mean2 += delta * ($1 - avg) } END { print sqrt(mean2 / NR) }'

This is a wonderful solution solving many of the problems with more naive
implementations:

=over 4

=item 1.

It's numerically well-behaved.

=item 2.

The subtractions guarantee that the computations will be done in the correct
type (i.e., I<double> instead of the type of the input).

=item 3.

Requires only two temporaries (!).

=item 4.

Requires only one pass over the data.

=back

I used to give the output array type I<float+>, but that creates more problems
than it solves. So now, standard deviations are always computed in type
I<double>. Besides being the default type in PDL and the 'natural'
floating-point type in C, it also makes the implementation easier.

=for bad

_istddev_loop processes bad values.
It will set the bad-value flag of all output ndarrays if the flag is set for any of the input ndarrays.


=cut






*_istddev_loop = \&PDL::_istddev_loop;





=head2 _istddev_post

=for sig

  Signature: (in(); indx count(); [o] out())

=for ref

Finalization for _istddev_loop().

=for bad

_istddev_post processes bad values.
The state of the bad-value flag of the output ndarrays is unknown.


=cut






*_istddev_post = \&PDL::_istddev_post;




=head1 AUTHOR

Edward Baudrez <ebaudrez@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2014 by Edward Baudrez.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut



;



# Exit with OK status

1;

		   